library(reshape2)
library(stringr)
library(lubridate)
library(plyr)
  
raw <- read.csv("billboard.csv", stringsAsFactors = F)
raw$date.peaked <- NULL
raw$artist.inverted <- iconv(raw$artist.inverted, "MAC", "UTF-8")
raw$track <- str_replace(raw$track, " \\(.*?\\)", "")
names(raw)[-(1:6)] <- str_c(1:76)

tidy <- melt(raw, 1:6, na.rm = T)
tidy$week <- as.integer(tidy$variable)
tidy$variable <- NULL

tidy$date.entered <- ymd(tidy$date.entered)
tidy$date <- tidy$date.entered + weeks(tidy$week - 1)
tidy$date.entered <- NULL

tidy <- rename(tidy, c("value" = "rank", "artist.inverted" = "artist"))
tidy <- tidy[c("year", "artist", "track", "time", "genre", "week", "date", "rank")]
tidy <- arrange(tidy, year, artist, track, week)

# Normalisation --------------------------------------------------------------

song <- unrowname(unique(tidy[c("artist", "track", "genre", "time")]))
song$id <- 1:nrow(song)

rank <- join(tidy, song, match = "first")
rank <- rank[c("id", "date", "rank")]
rank$date <- as.character(rank$date)

